/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef __GTK_ACL_H_

#include <gdk/gdk.h>
#include "gperms.h"

#define _(s) (gchar *) gettext (s)

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

	
#define GTK_TYPE_ACL          (gtk_acl_get_type ())
#define GTK_ACL(obj)          GTK_CHECK_CAST (obj, gtk_acl_get_type (), GtkAcl)
#define GTK_ACL_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, gtk_acl_get_type(), GtkAclClass)
#define IS_GTK_ACL(obj)       GTK_CHECK_TYPE (obj, gtk_acl_get_type ())
	

typedef struct _GtkAcl      GtkAcl;
typedef struct _GtkAclClass GtkAclClass;

//
struct _GtkAcl
{
	GtkHBox hbox;
  
	GtkTreeStore *acl_data;
	GtkWidget    *acl_view;
	
	GtkWidget *check_acl;
	GtkWidget *advanced ;
	GtkWidget *special[3];
	GtkWidget *special_frame;
	
	GtkWidget *win_usergroup;
	GtkWidget *win_usergroup_owner;
	

	GPerms *perms;
};

//
struct _GtkAclClass
{
	GtkHBoxClass parent_class;

	void (* add_acl) (GtkAcl *gtk_acl);
	void (* del_acl) (GtkAcl *gtk_acl);
};


GtkType    gtk_acl_get_type        (void);
GtkWidget *gtk_acl_new             (void);

/* End widget internal function */



void gtk_acl_load_file(GtkAcl *gtk_acl, const gchar *filename);

void gtk_acl_add_user(GtkAcl *gtk_acl, gpointer uid);
void gtk_acl_del_user(GtkAcl *gtk_acl, uid_t uid);
void gtk_acl_add_group(GtkAcl *gtk_acl, gpointer gid);
void gtk_acl_del_group(GtkAcl *gtk_acl, gid_t gid);

void gtk_acl_set_user_owner(GtkAcl *gtk_acl, gpointer gid);
void gtk_acl_set_group_owner(GtkAcl *gtk_acl, gpointer gid);

void gtk_acl_set_advanced_mode(GtkAcl *gtk_acl, const gboolean state);

#ifdef __cplusplus
}
#endif /* __cplusplus */
	
#endif
